﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CG2.Modeling;
using CG2.Rendering;
using CG2.Mathematics;
using CG2.Lighting;
using CG2.Shading;
using CG2.Texturing;

namespace CG2.RayTrace
{
    public partial class MainForm : Form
    {
		public static Random Rnd = new Random(0);

		private Camera camera;


		public MainForm()
		{
			InitializeComponent();

			cbScene.SelectedIndex = 0;
			cbAccelerator.SelectedIndex = 0;
			cbReflections.SelectedIndex = 1;
			cbRefractions.SelectedIndex = 1;
			cbAntiAlias.SelectedIndex = 0;
		}

		public void InitSceneRoom()
		{
			
			World world = new World();

			// TODO: uncomment after declarations
			/*#region Shaders

			Shader black = new PhongShader(new Vector4(0, 0, 0));
			Shader white = new PhongShader(new Vector4(1, 1, 0));
			Shader red = new PhongShader(new Vector4(1, 0, 0));
			Shader green = new PhongShader(new Vector4(0, 1, 0));
			Shader blue = new PhongShader(new Vector4(0, 0, 0.5));
			Shader mirror = new PhongShader(new Vector4(0.1, 0.1, 0.1), new Vector4(0.1, 0.1, 0.1)) { ReflectionFactor = 1.0, RefractionFactor = 0.0, RefractionIndex = 2.00 };
			Shader glass = new PhongShader(new Vector4(0.1, 0.1, 0.1), new Vector4(0.1, 0.1, 0.1)) { ReflectionFactor = 1.0, RefractionFactor = 0.9, RefractionIndex = 1.45 };

			Shader greenNoise = new PhongShader()
			{
				DiffuseSampler = new ColorSampler(new Vector4(0, 1, 0)),
				NormalSampler = new NoiseSampler() { Magnitude = 0.02 },
				IsDeltaNormalSampler = true,
				ReflectionFactor = 0.8,
			};

			Shader checker = new CheckerShader()
			{
				Shader0 = blue,
				Shader1 = white,
			};

			SphereMapping sphereMapping = new SphereMapping(new Vector4(6, 9, 2), Vector4.UnitX, Vector4.UnitY);
			Shader homer = new PhongShader()
			{
				DiffuseSampler = new Texture2Sampler("Content/homer.png", sphereMapping, TextureFiltering.Nearest, TextureWrap.Clamp, TextureWrap.Clamp)
			};

			PlaneMapping wallMapY0 = new PlaneMapping(new Vector4(0, 0, 0), new Vector4(-10, 0, 0), new Vector4(0, 0, 10)) { Offset = new Vector4(0.2, 0.2, 0) };
			Shader wallY0 = new PhongShader()
			{
				DiffuseSampler = new BlendingSampler()
				{
					Blending = BlendingOperation.Mul,
					Sampler0 = new Texture2Sampler("Content/wall-diffuse.png", wallMapY0, TextureFiltering.Bilinear, TextureWrap.Repeat, TextureWrap.Repeat),
					Sampler1 = new ColorSampler(new Vector4(1.5, 1.5, 1.5)),
				},
				SpecularSampler = new ColorSampler(new Vector4(0.4, 0.4, 0.4)),
				NormalSampler = new NormalTexture2Sampler("Content/wall-normal.png", wallMapY0, TextureFiltering.Bilinear, TextureWrap.Repeat, TextureWrap.Repeat),
			};

			PlaneMapping wallMapY1 = new PlaneMapping(new Vector4(0, 0, 0), new Vector4(-10, 0, 0), new Vector4(0, 0, 10)) { Offset = new Vector4(0.2, 0.2, 0) };
			Shader wallY1 = new PhongShader()
			{
				DiffuseSampler = new BlendingSampler()
				{
					Blending = BlendingOperation.Mul,
					Sampler0 = new Texture2Sampler("Content/wall-diffuse.png", wallMapY0, TextureFiltering.Bilinear, TextureWrap.Repeat, TextureWrap.Repeat),
					Sampler1 = new ColorSampler(new Vector4(1.5, 1.5, 1.5)),
				},
				SpecularSampler = new ColorSampler(new Vector4(0.4, 0.4, 0.4)),
			};

			Shader homerX0 = new PhongShader()
			{
				DiffuseSampler = new BlendingSampler()
				{
					Blending = BlendingOperation.Alpha1,
					Sampler0 = new BlendingSampler()
					{
						Blending = BlendingOperation.Alpha1,
						Sampler0 = new ColorSampler(new Vector4(1, 0, 0)),
						Sampler1 = new Texture2Sampler("Content/homer.png", null, TextureFiltering.Bilinear, TextureWrap.Repeat, TextureWrap.Repeat)
						{
							Mapping = new PlaneMapping(new Vector4(0, 0, 0), new Vector4(0, -10, 0), new Vector4(0, 0, -10))
						}
					},
					Sampler1 = new Texture2Sampler("Content/wanted.png", null, TextureFiltering.Bilinear, TextureWrap.Repeat, TextureWrap.Repeat)
					{
						Mapping = new PlaneMapping(new Vector4(0, 0, 0), new Vector4(0, 9, 0), new Vector4(0, 0, -9))
					}
				},
			};

			#endregion

			#region Models

			world.Models.Add(new Plane(homerX0, new Vector4(0, 0, 0), new Vector4(1, 0, 0)));
			world.Models.Add(new Plane(wallY0, new Vector4(0, 0, 0), new Vector4(0, 1, 0)));
			world.Models.Add(new Plane(checker, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));
			world.Models.Add(new Plane(wallY1, new Vector4(0, 10, 0), new Vector4(0, -1, 0)));
			world.Models.Add(new Plane(white, new Vector4(0, 0, 8), new Vector4(0, 0, -1)));

			// world.Models.Add(new Sphere(glass, new Vector4(6, 9, 2), 2));
			world.Models.Add(new Sphere(homer, new Vector4(6, 9, 2), 2));
			// world.Models.Add(new Sphere(greenNoise, new Vector4(6, 9, 2), 2));

			world.Models.Add(new AABB(mirror, new Vector4(5, 1, 1), new Vector4(10, 4, 2)));
			// world.Models.Add(new AABB(red, new Vector4(0, 8, 0), new Vector4(5, 10, 2)));
			// world.Models.Add(new AABB(glass, new Vector4(5, 6, 0.5), new Vector4(7, 9, 6)));

			Mesh mesh = new Mesh(greenNoise);
			mesh.SetPyramid(new Vector4(3, 3, 1), new Vector4(5, 5, 1), new Vector4(1, 7, 1), new Vector4(2, 4, 5));
			world.Models.Add(mesh);

			#endregion

			#region Lights

			PointLight point = new PointLight()
			{
				Origin = new Vector4(7, 5, 6),
				AmbientColor = new Vector4(0.1, 0.1, 0.1),
				Intensity = 1.2,
				Range = 7.0,
			};
			world.Lights.Add(point);

			#endregion*/

			camera = new Camera(500, 400)
			{
				Position = new Vector4(20, 5, 4.5),
				Target = new Vector4(0, 5, 4.5),
				World = world,
			};
			 
		}

		public void InitSceneSpheres()
		{
			World world = new World();

			Shader red = new PhongShader(new Vector4(1, 0, 0));

			for (Int32 i = 0; i < 200; i++)
			{
				Sphere sphere = new Sphere()
				{
					Origin = new Vector4(100 * Rnd.NextDouble(), 100 * Rnd.NextDouble(), 100 * Rnd.NextDouble()),
					Radius = 2.0,
					Shader = red,
				};
				world.Models.Add(sphere);
			}

			SunLight sun = new SunLight()
			{
				Direction = new Vector4(-1, -1, -1),
				Intensity = 1,
			};
			world.Lights.Add(sun);

			camera = new Camera(500, 400)
			{
				Position = new Vector4(200, 50, 50),
				Target = new Vector4(0, 50, 50),
				World = world,
			};
		}

		public void InitSceneLight(String lightType)
		{
			World world = new World();

			Shader white = new PhongShader(new Vector4(1, 1, 1));
			Shader gray = new PhongShader(new Vector4(0.5, 0.5, 0.5));
			Shader red = new PhongShader(new Vector4(1, 0, 0));

			Shader checker = new CheckerShader()
			{
				Shader0 = gray,
				Shader1 = white,
				CubeSize = 4
			};

			world.Models.Add(new Plane(checker, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));
			world.Models.Add(new Sphere(red, new Vector4(0, 0, 4), 2));

			SunLight sun = new SunLight()
			{
				Intensity = 1.0,
				Direction = new Vector4(-5, -5, -10)
			};

			PointLight point = new PointLight()
			{
				Intensity = 7.0,
				Origin = new Vector4(5, 5, 10),
				Range = 7.0,
				LinearAttenuation = 1,
				QuadraticAttenuation = 1
			};

			SpotLight spot = new SpotLight()
			{
				Intensity = 7.0,
				Origin = new Vector4(5, 5, 10),
				Direction = new Vector4(-5, -5, -10),
				Range = 7.0,
				LinearAttenuation = 1,
				QuadraticAttenuation = 1,
				CutoffDeg = 45,
				Exp = 1,
			};

			switch (lightType)
			{
				case "SunLight": world.Lights.Add(sun); break;
				case "PointLight": world.Lights.Add(point); break;
				case "SpotLight": world.Lights.Add(spot); break;
			}

			camera = new Camera(500, 400)
			{
				Position = new Vector4(30, -10, 10),
				Target = new Vector4(0, 0, 0),
				World = world,
			};
		}

		public void InitSceneReflections()
		{
			World world = new World();

			#region Shaders

			Shader red = new PhongShader(new Vector4(0.5, 0, 0));
			Shader white = new PhongShader(new Vector4(1, 1, 1));
			Shader blue = new PhongShader(new Vector4(0, 0, 1));

			Shader checker = new CheckerShader() { Shader0 = white, Shader1 = red, CubeSize = 1 };

			Shader mirror = new PhongShader(new Vector4(0.0, 0.0, 0.0), new Vector4(0.1, 0.1, 0.1)) { ReflectionFactor = 0.9, RefractionFactor = 0.0, RefractionIndex = 2.00 };

			#endregion

			#region Models

			world.Models.Add(new Plane(mirror, new Vector4(0, 0, 0), new Vector4(1, 0, 0)));
			world.Models.Add(new Plane(mirror, new Vector4(40, 0, 0), new Vector4(-1, 0, 0)));
			world.Models.Add(new Plane(checker, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));

			world.Models.Add(new Sphere(blue, new Vector4(15, 3, 2), 1));

			#endregion

			#region Lights

			PointLight point = new PointLight()
			{
				Origin = new Vector4(15, 5, 10),
				AmbientColor = new Vector4(0.0, 0.0, 0.0),
				Intensity = 1.0,
				Range = 20.0,
				LinearAttenuation = 0.2,
				QuadraticAttenuation = 0
			};
			world.Lights.Add(point);

			#endregion

			camera = new Camera(500, 400)
			{
				Position = new Vector4(30, 10, 7),
				Target = new Vector4(0, 5, 4),
				World = world,
			};
		}

		public void InitSceneRefractions()
		{
			World world = new World();

			#region Shaders

			Shader black = new PhongShader(new Vector4(0, 0, 0.5));
			Shader white = new PhongShader(new Vector4(1, 1, 1));

			Shader checker = new CheckerShader() { Shader0 = white, Shader1 = black, CubeSize = 1 };

			Shader glass0 = new PhongShader(new Vector4(0.5, 0.1, 0.1), new Vector4(0.1, 0.1, 0.1)) { RefractionFactor = 0.8, RefractionIndex = 1.00 };
			Shader glass1 = new PhongShader(new Vector4(0.5, 0.1, 0.1), new Vector4(0.1, 0.1, 0.1)) { RefractionFactor = 0.8, RefractionIndex = 1.05 };
			Shader glass2 = new PhongShader(new Vector4(0.5, 0.1, 0.1), new Vector4(0.1, 0.1, 0.1)) { RefractionFactor = 0.8, RefractionIndex = 1.10 };
			Shader glass3 = new PhongShader(new Vector4(0.5, 0.1, 0.1), new Vector4(0.1, 0.1, 0.1)) { RefractionFactor = 0.8, RefractionIndex = 1.15 };
			Shader glass4 = new PhongShader(new Vector4(0.5, 0.1, 0.1), new Vector4(0.1, 0.1, 0.1)) { RefractionFactor = 0.8, RefractionIndex = 1.20 };
			Shader glass5 = new PhongShader(new Vector4(0.5, 0.1, 0.1), new Vector4(0.1, 0.1, 0.1)) { RefractionFactor = 0.8, RefractionIndex = 1.25 };

			Shader glass6 = new PhongShader(new Vector4(0.1, 0.5, 0.1), new Vector4(0.1, 0.1, 0.1)) { RefractionFactor = 0.8, RefractionIndex = 1.40 };

			#endregion

			#region Models

			world.Models.Add(new Plane(checker, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));

			world.Models.Add(new Sphere(glass0, new Vector4(0, 0, 2), 1.2));
			world.Models.Add(new Sphere(glass1, new Vector4(0, 3, 2), 1.2));
			world.Models.Add(new Sphere(glass2, new Vector4(0, 6, 2), 1.2));
			world.Models.Add(new Sphere(glass3, new Vector4(0, 9, 2), 1.2));
			world.Models.Add(new Sphere(glass4, new Vector4(0, 12, 2), 1.2));
			world.Models.Add(new Sphere(glass5, new Vector4(0, 15, 2), 1.2));

			world.Models.Add(new AABB(glass6, new Vector4(3, 5, 0), new Vector4(4, 13, 8)));

			#endregion

			#region Lights

			PointLight point = new PointLight()
			{
				Origin = new Vector4(5, 5, 15),
				AmbientColor = new Vector4(0.0, 0.0, 0.0),
				Intensity = 1.2,
				Range = 50.0,
				LinearAttenuation = 0.2,
				QuadraticAttenuation = 0
			};
			world.Lights.Add(point);

			#endregion

			camera = new Camera(500, 400)
			{
				Position = new Vector4(18, 15, 10),
				Target = new Vector4(0, 8, 3),
				World = world,
			};
		}

		public void InitSceneInternalReflection()
		{
			World world = new World();

			#region Shaders

			Shader black = new PhongShader(new Vector4(0, 0.5, 0));
			Shader white = new PhongShader(new Vector4(1, 1, 1));
			Shader red = new PhongShader(new Vector4(1, 0, 0));

			Shader checker = new CheckerShader() { Shader0 = white, Shader1 = black, CubeSize = 1 };

			Shader glass = new PhongShader(new Vector4(0.1, 0.1, 0.1), new Vector4(0.1, 0.1, 0.1)) { RefractionFactor = 0.7, RefractionIndex = 0.92 };

			#endregion

			#region Models

			world.Models.Add(new Plane(checker, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));

			world.Models.Add(new Sphere(red, new Vector4(0, -5, 5), 1.2));
			world.Models.Add(new Sphere(red, new Vector4(-10, -5, 5), 1.2));
			world.Models.Add(new Sphere(red, new Vector4(-20, -5, 5), 1.2));

			world.Models.Add(new AABB(glass, new Vector4(-70, -20, 1), new Vector4(10, 10, 1.5)));

			#endregion

			#region Lights

			PointLight point = new PointLight()
			{
				Origin = new Vector4(0, 10, 20),
				AmbientColor = new Vector4(0.0, 0.0, 0.0),
				Intensity = 1.0,
				Range = 100.0,
				LinearAttenuation = 0.2,
				QuadraticAttenuation = 0
			};
			world.Lights.Add(point);

			#endregion

			camera = new Camera(500, 400)
			{
				Position = new Vector4(20, 20, 15),
				Target = new Vector4(0, 5, 3),
				World = world,
				BgColor = new Vector4(0.4, 0.4, 0.9),
			};
		}

		public void InitSceneSampling()
		{
			World world = new World();

			// TODO: uncomment after declarations
			/*#region Shaders

			Shader homerX0 = new PhongShader()
			{
				DiffuseSampler = new Texture2Sampler("Content/homer.png", null, TextureFiltering.Nearest, TextureWrap.Mirror, TextureWrap.Mirror)
				{
					Mapping = new PlaneMapping(new Vector4(0, 0, 0), 2 * Vector4.UnitY, 2 * Vector4.UnitZ)
				}
			};

			Shader homerY0 = new PhongShader()
			{
				DiffuseSampler = new Texture2Sampler("Content/homer.png", null, TextureFiltering.Nearest, TextureWrap.Clamp, TextureWrap.Clamp)
				{
					Mapping = new PlaneMapping(new Vector4(0, 0, 0), 15 * Vector4.UnitX, 15 * Vector4.UnitZ)
				}
			};

			Shader homerZ0 = new PhongShader()
			{
				DiffuseSampler = new Texture2Sampler("Content/homer.png", null, TextureFiltering.Nearest, TextureWrap.Mirror, TextureWrap.Repeat)
				{
					Mapping = new PlaneMapping(new Vector4(0, 0, 0), 3 * Vector4.UnitX, 3 * Vector4.UnitY)
				}
			};


			Shader homerY1 = new PhongShader()
			{
				DiffuseSampler = new Texture2Sampler("Content/homer.png", null, TextureFiltering.Bilinear, TextureWrap.Clamp, TextureWrap.Clamp)
				{
					Mapping = new PlaneMapping(new Vector4(0, 10, 0), 15 * Vector4.UnitX, 15 * Vector4.UnitZ)
				}
			};

			Shader homerZ1 = new PhongShader()
			{
				DiffuseSampler = new Texture2Sampler("Content/homer.png", null, TextureFiltering.Nearest, TextureWrap.Clamp, TextureWrap.Clamp)
				{
					Mapping = new PlaneMapping(new Vector4(0, 0, 8), 5 * Vector4.UnitX, 5 * Vector4.UnitY),
				}
			};

			#endregion

			#region Models

			world.Models.Add(new Plane(homerX0, new Vector4(0, 0, 0), new Vector4(1, 0, 0)));
			world.Models.Add(new Plane(homerY0, new Vector4(0, 0, 0), new Vector4(0, 1, 0)));
			world.Models.Add(new Plane(homerZ0, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));
			world.Models.Add(new Plane(homerY1, new Vector4(0, 10, 0), new Vector4(0, -1, 0)));
			world.Models.Add(new Plane(homerZ1, new Vector4(0, 0, 8), new Vector4(0, 0, -1)));

			#endregion

			#region Lights

			PointLight point = new PointLight()
			{
				Origin = new Vector4(7, 5, 6),
				AmbientColor = new Vector4(0.1, 0.1, 0.1),
				Intensity = 1.2,
				Range = 7.0,
			};
			world.Lights.Add(point);

			#endregion
			*/

			camera = new Camera(500, 400)
			{
				Position = new Vector4(20, 5, 4.5),
				Target = new Vector4(0, 5, 4.5),
				World = world,
			};
		}

		public void InitSceneMapping()
		{
			World world = new World();

			// TODO: uncomment after declarations
			/*#region Shaders

			Shader red = new PhongShader(new Vector4(0.5, 0, 0));

			Shader checker = new PhongShader()
			{
				DiffuseSampler = new Texture2Sampler("Content/checker.png", null, TextureFiltering.Nearest, TextureWrap.Repeat, TextureWrap.Repeat)
				{
					Mapping = new SphereMapping(new Vector4(15, 3, 4), Vector4.UnitX, Vector4.UnitY) { Scale = new Vector4(0.4, 0.4, 1) }
				}
			};

			#endregion


			#region Models

			world.Models.Add(new Plane(red, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));

			world.Models.Add(new Sphere(checker, new Vector4(15, 3, 4), 4));

			#endregion


			#region Lights

			PointLight point = new PointLight()
			{
				Origin = new Vector4(30, 0, 20),
				//SpecularColor = new Vector4(0,0,0), 
				AmbientColor = new Vector4(0.3, 0.3, 0.3),
				Intensity = 1.0,
				Range = 20.0,
				LinearAttenuation = 0.2,
				QuadraticAttenuation = 0
			};
			world.Lights.Add(point);

			#endregion
			*/

			camera = new Camera(500, 400)
			{
				Position = new Vector4(30, 5, 15),
				Target = new Vector4(10, 5, 2),
				World = world,
			};
		}

		public void InitSceneBlending()
		{
			World world = new World();

			// TODO: uncomment after declarations
			/*#region Shaders

			Shader black = new PhongShader(new Vector4(0, 0.5, 0));
			Shader white = new PhongShader(new Vector4(1, 1, 1));
			Shader red = new PhongShader(new Vector4(1, 0, 0));

			Shader checker = new CheckerShader() { Shader0 = white, Shader1 = black, CubeSize = 1 };

			Shader homerZ0 = new PhongShader()
			{
				DiffuseSampler = new BlendingSampler()
				{
					Blending = BlendingOperation.Add,
					Sampler0 = new ColorSampler(new Vector4(0.4, 0, 0)),
					Sampler1 = new BlendingSampler()
					{
						Blending = BlendingOperation.Mul,
						Sampler0 = new Texture2Sampler("Content/mask.png", null, TextureFiltering.Bilinear, TextureWrap.Repeat, TextureWrap.Repeat)
						{
							Mapping = new PlaneMapping(new Vector4(10, 10, 0), 42 * Vector4.UnitY, -42 * Vector4.UnitZ) { Offset = new Vector4(-0.25, -0.6, 0) }
						},
						Sampler1 = new Texture2Sampler("Content/homer.png", null, TextureFiltering.Nearest, TextureWrap.Mirror, TextureWrap.Repeat)
						{
							Mapping = new PlaneMapping(new Vector4(10, 10, 0), 15 * Vector4.UnitY, -15 * Vector4.UnitZ)
						}
					},
				},
			};

			#endregion

			#region Models

			world.Models.Add(new Plane(checker, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));

			world.Models.Add(new AABB(homerZ0, new Vector4(1, -20, 0), new Vector4(3, 20, 15)));

			#endregion

			#region Lights

			PointLight point = new PointLight()
			{
				Origin = new Vector4(10, 10, 15),
				AmbientColor = new Vector4(0.0, 0.0, 0.0),
				Intensity = 1.0,
				Range = 100.0,
				LinearAttenuation = 0.2,
				QuadraticAttenuation = 0
			};
			world.Lights.Add(point);

			#endregion
			*/

			camera = new Camera(500, 400)
			{
				Position = new Vector4(25, 25,  20),
				Target = new Vector4(0, 5, 3),
				World = world,
				BgColor = new Vector4(0.4, 0.4, 0.9),
			};
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Graphics g = e.Graphics;

			if (camera.Bitmap != null) g.DrawImage(camera.Bitmap, 0, 0);
		}


		private Int32[] reflectionCounts = { 0, 1, 2, 3, 4, 5, 10, 50 };
		private Int32[] refractionCounts = { 0, 1, 2, 3, 4, 5, 10, 50 };

		private void bRender_Click(object sender, EventArgs e)
		{
			camera.UseShadows = cbShadows.Checked;
			camera.UseFresnel = cbFresnel.Checked;

			camera.ReflectionCount = reflectionCounts[cbReflections.SelectedIndex];
			camera.RefractionCount = refractionCounts[cbRefractions.SelectedIndex];

			Cursor = Cursors.WaitCursor;
			DateTime t0 = DateTime.Now;
			camera.Render();
			DateTime t1 = DateTime.Now;
			Cursor = Cursors.Default;

			lRenderTime.Text = "Render: " + (t1 - t0).TotalMilliseconds.ToString("F0") + " ms";

			Invalidate();
		}

		private void cbScene_SelectedIndexChanged(object sender, EventArgs e)
		{
			switch ((String)cbScene.SelectedItem)
			{
				case "Room": InitSceneRoom(); break;
				case "Spheres": InitSceneSpheres(); break;

				case "Sun Light": InitSceneLight(typeof(SunLight).Name); break;
				case "Point Light": InitSceneLight(typeof(PointLight).Name); break;
				case "Spot Light": InitSceneLight(typeof(SpotLight).Name); break;

				case "Reflections": InitSceneReflections(); break;
				case "Refractions": InitSceneRefractions(); break;

				case "Internal Reflection": InitSceneInternalReflection(); break;

				case "Sampling": InitSceneSampling(); break;
				case "Mapping": InitSceneMapping(); break;

				case "Blending": InitSceneBlending(); break;
			}
		}

		private void cbAccelerator_SelectedIndexChanged(object sender, EventArgs e)
		{
			switch ((String)cbAccelerator.SelectedItem)
			{
				case "NoAccelerator": camera.Accelerator = new RayAccelerator(); break;
				case "Grid": camera.Accelerator = new GridAccelerator(); break;
				case "Bsp": camera.Accelerator = new BspAccelerator(); break;
			}
		}

		private void cbAntiAlias_SelectedIndexChanged(object sender, EventArgs e)
		{
			switch ((String)cbAntiAlias.SelectedItem)
			{
				// TODO: uncomment after declarations
				/*case "No AntiAliasing": camera.SubSampleCount = 1; break;
				case "2x2 AntiAliasing": camera.SubSampleCount = 2; break;
				case "4x4 AntiAliasing": camera.SubSampleCount = 4; break;*/
			}
		}

    }
}
