﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CG2.Mathematics;
using CG2.Modeling;
using CG2.Shading;

namespace CG2.Rendering
{
	public class Ray
	{
		#region Properties

		public static Double Bias = 0.01;

		public Vector4 Origin;
		public Vector4 Direction;

		public Double HitParam = Double.MaxValue;
		public Vector4 HitNormal;
		public Model HitModel = null;

		public Model InModel = null; // transparent model in which this ray travels (usually previous HitModel)

		public Int32 ReflectionLevel = 0;
		public Int32 RefractionLevel = 0;

		#endregion


		#region Init

		public Ray()
		{
		}

		public Ray(Vector4 origin, Vector4 direction, Model inModel, Int32 reflectionLevel, Int32 refractionLevel)
		{
			Set(origin, direction, inModel, reflectionLevel, refractionLevel);
		}

		public void Set(Vector4 origin, Vector4 direction, Model inModel, Int32 reflectionLevel, Int32 refractionLevel)
		{
			Origin = origin;
			Direction = direction;
			HitParam = Double.MaxValue;
			InModel = inModel;
			ReflectionLevel = reflectionLevel;
			RefractionLevel = refractionLevel;
		}

		#endregion


		#region Geometry

		public Vector4 GetHitPoint()
		{
			return Origin + HitParam * Direction;
		}

		#endregion
	}
}
