﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CG2.Mathematics;

namespace CG2.Texturing
{
	public class SphereMapping : TextureMapping
	{
		#region Properties

		public Vector4 Origin = Vector4.Zero;
		public Vector4 X = Vector4.UnitX;
		public Vector4 Y = Vector4.UnitY;
		public Vector4 Z = Vector4.UnitZ;
		public Vector4 Scale = new Vector4(1, 1, 1);

		#endregion


		#region Init

		public SphereMapping(Vector4 origin, Vector4 x, Vector4 y)
		{
			Origin = origin;
			X = x.Normalized;
			Z = (x % y).Normalized;
			Y = Z % X; // |X| = 1 and |Z| = 1 -> |Y| = 1
		}

		#endregion


		#region Mapping

		public override void GetTangetSpace(Vector4 point, out Vector4 u, out Vector4 v, out Vector4 w)
		{
			// TODO: set u,v,w being base vectors for the tangent space
			u = Vector4.Zero;
			v = Vector4.Zero;
			w = Vector4.Zero;
		}

		public override Vector4 GetCoords(Vector4 point)
		{
			Vector4 r = point - Origin;
			Double l = r.Length;
			r = new Vector4(r * X, r * Y, r * Z); // project r into local X,Y,Z space

			Vector4 rXY = new Vector4(r.X, r.Y, 0);
			Double lXY = rXY.Length;

			Double theta = Math.Atan2(r.Y, r.X) + Math.PI;
			Double phi = Math.Atan2(r.Z, lXY) + Math.PI / 2;

			return Offset + new Vector4(
				theta / (2 * Math.PI) / Scale.X,
				phi / Math.PI / Scale.Y,
				l / Scale.Z);
		}

		#endregion
	}
}
