﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CG2.Mathematics;
using System.Drawing;

namespace CG2.Texturing
{
	public enum TextureWrap
	{
		Clamp,
		Repeat,
		Mirror
	}

	public enum TextureFiltering
	{
		Nearest,
		Bilinear,
	}

	public class Texture2Sampler : Sampler
	{
		#region Properties

		public Bitmap Bitmap;

		// TODO: declare Mapping for this sampler
		public TextureFiltering Filtering = TextureFiltering.Nearest;
		public TextureWrap WrapU = TextureWrap.Clamp;
		public TextureWrap WrapV = TextureWrap.Clamp;

		#endregion


		#region Init

		public Texture2Sampler()
		{
		}

		public Texture2Sampler(String bitmapFile, TextureMapping mapping, TextureFiltering filtering, TextureWrap wrapU, TextureWrap wrapV)
		{
			Bitmap = new Bitmap(bitmapFile);
			// TODO: set given mapping
			Filtering = filtering;
			WrapU = wrapU;
			WrapV = wrapV;
		}

		#endregion


		#region Sampler

		public override Vector4 GetSample(Vector4 point)
		{
			// TODO: get sampling coordinates using mapping
			Vector4 uvw = Vector4.Zero;

			switch (WrapU)
			{
				case TextureWrap.Clamp:
					if (uvw.X < 0) uvw.X = 0; else if (uvw.X > 1) uvw.X = 1;
					break;

				case TextureWrap.Repeat:
					uvw.X -= Math.Floor(uvw.X);
					break;

				case TextureWrap.Mirror:
					Int32 f = (Int32)Math.Floor(uvw.X);
					uvw.X -= f;
					if ((f & 1) == 1) uvw.X = 1 - uvw.X;
					break;
			}

			switch (WrapV)
			{
				case TextureWrap.Clamp:
					if (uvw.Y < 0) uvw.Y = 0; else if (uvw.Y > 1) uvw.Y = 1;
					break;

				case TextureWrap.Repeat:
					uvw.Y -= Math.Floor(uvw.Y);
					break;

				case TextureWrap.Mirror:
					Int32 f = (Int32)Math.Floor(uvw.Y);
					uvw.Y -= f;
					if ((f & 1) == 1) uvw.Y = 1 - uvw.Y;
					break;
			}

			switch (Filtering)
			{
				case TextureFiltering.Nearest:
					Color c = Bitmap.GetPixel(
						(Int32)(uvw.X * (Bitmap.Width - 2)),
						(Int32)(uvw.Y * (Bitmap.Height - 2))
					);
					// TODO: store alpha chanel in W component
					return new Vector4(c.R / 255.0, c.G / 255.0, c.B / 255.0);

				case TextureFiltering.Bilinear:
					Double u = uvw.X * (Bitmap.Width - 2);
					Double v = uvw.Y * (Bitmap.Height - 2);
					Double u1 = u - (Int32)u;
					Double v1 = v - (Int32)v;
					Double u0 = 1 - u1;
					Double v0 = 1 - v1;

					Color c00 = Bitmap.GetPixel((Int32)u + 0, (Int32)v + 0);
					Color c01 = Bitmap.GetPixel((Int32)u + 0, (Int32)v + 1);
					Color c10 = Bitmap.GetPixel((Int32)u + 1, (Int32)v + 0);
					Color c11 = Bitmap.GetPixel((Int32)u + 1, (Int32)v + 1);

					// TODO: store interpolated alpha chanel in W component
					return new Vector4(
						((c00.R / 255.0) * u0 + (c10.R / 255.0) * u1) * v0 + ((c01.R / 255.0) * u0 + (c11.R / 255.0) * u1) * v1,
						((c00.G / 255.0) * u0 + (c10.G / 255.0) * u1) * v0 + ((c01.G / 255.0) * u0 + (c11.G / 255.0) * u1) * v1,
						((c00.B / 255.0) * u0 + (c10.B / 255.0) * u1) * v0 + ((c01.B / 255.0) * u0 + (c11.B / 255.0) * u1) * v1);

			}

			return Vector4.Zero;
		}

		#endregion
	}
}
